@@ -1,7 +1,6 @@
 """Portfolio Manager specialist module."""
 
 from examples.portfolio_analysis.specialists.portfolio.models import AllocationChange, PortfolioRecommendations
-from examples.portfolio_analysis.specialists.portfolio.node import PortfolioManager
 from examples.portfolio_analysis.specialists.portfolio.signature import PortfolioManagerSignature
 
-__all__ = ["AllocationChange", "PortfolioManager", "PortfolioManagerSignature", "PortfolioRecommendations"]
+__all__ = ["AllocationChange", "PortfolioManagerSignature", "PortfolioRecommendations"]
