@@ -16,7 +16,7 @@ import ast
 import re
 import sys
 from pathlib import Path
-from typing import NamedTuple
+from typing import NamedTuple, cast
 
 
 class Violation(NamedTuple):
@@ -40,7 +40,7 @@ def _check_line_for_suppression(line: str, pattern: str) -> bool:
     return bool(re.search(pattern, line, re.IGNORECASE))
 
 
-def _get_lines_to_check(lines: list[str], line_num: int) -> list[str]:
+def _get_lines_to_check(lines: tuple[str, ...], line_num: int) -> tuple[str, ...]:
     """Get the current line and next 2 lines for suppression checking.
 
     Returns:
@@ -48,7 +48,7 @@ def _get_lines_to_check(lines: list[str], line_num: int) -> list[str]:
 
     """
     if line_num <= 0 or line_num > len(lines):
-        return []
+        return ()
     # Get current line and up to 2 following lines
     start_idx = line_num - 1
     end_idx = min(start_idx + 3, len(lines))
@@ -70,7 +70,7 @@ def has_suppression(content: str, line_num: int, code: str) -> bool:
         # clearflow: ignore[ARCH001]  - Specific code suppression
 
     """
-    lines = content.splitlines()
+    lines = tuple(content.splitlines())
     lines_to_check = _get_lines_to_check(lines, line_num)
     if not lines_to_check:
         return False
@@ -80,17 +80,47 @@ def has_suppression(content: str, line_num: int, code: str) -> bool:
     return any(_check_line_for_suppression(line, pattern) for line in lines_to_check)
 
 
-def _check_private_imports(node: ast.ImportFrom, file_path: Path, *, is_internal: bool) -> Violation | None:
+def _check_private_imports(node: ast.ImportFrom, file_path: Path, *, is_internal: bool) -> tuple[Violation, ...]:
     """Check for imports from private implementation modules.
 
     Returns:
-        Violation if private import found, None otherwise.
+        Tuple of violations found.
+
+    """
+    violations = ()
+    if not node.module:
+        return tuple(violations)
+
+    # Check for _internal module imports
+    violation = _check_internal_imports(node, file_path, is_internal=is_internal)
+    if violation:
+        violations = (*violations, violation)
+
+    # Check for test files importing from non-public API modules
+    violation = _check_test_imports(node, file_path)
+    if violation:
+        violations = (*violations, violation)
+
+    return violations
+
+
+def _check_internal_imports(node: ast.ImportFrom, file_path: Path, *, is_internal: bool) -> Violation | None:
+    """Check for imports from _internal modules.
+
+    Returns:
+        Violation if found, None otherwise.
 
     """
     if not node.module:
         return None
+
+    # Public wrapper modules are allowed to import from _internal
+    public_wrapper_modules = {"__init__.py"}
+    is_public_wrapper = file_path.name in public_wrapper_modules and "_internal" not in str(file_path)
+
+    # Check for _internal module imports
     private_module = "clearflow." + "_internal"
-    if private_module in node.module and not is_internal:
+    if private_module in node.module and not is_internal and not is_public_wrapper:
         return Violation(
             file=file_path,
             line=node.lineno,
@@ -102,6 +132,36 @@ def _check_private_imports(node: ast.ImportFrom, file_path: Path, *, is_internal
     return None
 
 
+def _check_test_imports(node: ast.ImportFrom, file_path: Path) -> Violation | None:
+    """Check for test files importing from non-public API modules.
+
+    Returns:
+        Violation if found, None otherwise.
+
+    """
+    is_test_file = "test" in str(file_path) or file_path.name.startswith("conftest")
+    if not is_test_file or not node.module or not node.module.startswith("clearflow."):
+        return None
+
+    # List of non-public modules that tests should not import from directly
+    non_public_modules = [
+        "clearflow.message",
+        "clearflow.message_node",
+        "clearflow.message_flow",
+        "clearflow.observer",
+    ]
+    if node.module in non_public_modules:
+        return Violation(
+            file=file_path,
+            line=node.lineno,
+            column=node.col_offset,
+            code="ARCH011",
+            message=f"Test file importing from non-public API module '{node.module}' - tests must use only public API from clearflow.__init__.py",
+            requirement="REQ-ARCH-011",
+        )
+    return None
+
+
 def _check_mock_imports(node: ast.ImportFrom, file_path: Path) -> tuple[Violation, ...]:
     """Check for unittest.mock imports that violate architecture.
 
@@ -109,12 +169,13 @@ def _check_mock_imports(node: ast.ImportFrom, file_path: Path) -> tuple[Violatio
         Tuple of violations found.
 
     """
-    violations: list[Violation] = []
+    violations = ()
     if node.module == "unittest.mock":
         for alias in node.names:
             name = alias.name
             if name in {"patch", "Mock", "MagicMock"}:
-                violations.append(
+                violations = (
+                    *violations,
                     Violation(
                         file=file_path,
                         line=node.lineno,
@@ -124,7 +185,7 @@ def _check_mock_imports(node: ast.ImportFrom, file_path: Path) -> tuple[Violatio
                         requirement="REQ-ARCH-002",
                     ),
                 )
-    return tuple(violations)
+    return violations
 
 
 def _check_typing_imports(node: ast.ImportFrom, file_path: Path, content: str) -> tuple[Violation, ...]:
@@ -134,14 +195,15 @@ def _check_typing_imports(node: ast.ImportFrom, file_path: Path, content: str) -
         Tuple of violations found.
 
     """
-    violations: list[Violation] = []
+    violations = ()
     if not node.module:
         return tuple(violations)
 
     if node.module == "typing" or node.module.startswith("typing"):
         for alias in node.names:
             if alias.name == "TYPE_CHECKING" and not has_suppression(content, node.lineno, "ARCH008"):
-                violations.append(
+                violations = (
+                    *violations,
                     Violation(
                         file=file_path,
                         line=node.lineno,
@@ -151,8 +213,9 @@ def _check_typing_imports(node: ast.ImportFrom, file_path: Path, content: str) -
                         requirement="REQ-ARCH-008",
                     ),
                 )
-            elif alias.name == "Any":
-                violations.append(
+            elif alias.name == "Any" and not has_suppression(content, node.lineno, "ARCH010"):
+                violations = (
+                    *violations,
                     Violation(
                         file=file_path,
                         line=node.lineno,
@@ -162,7 +225,7 @@ def _check_typing_imports(node: ast.ImportFrom, file_path: Path, content: str) -
                         requirement="REQ-ARCH-010",
                     ),
                 )
-    return tuple(violations)
+    return violations
 
 
 def _check_type_checking_block(node: ast.If, file_path: Path, content: str) -> Violation | None:
@@ -197,7 +260,7 @@ def _check_object_in_params(
         Tuple of violations found.
 
     """
-    violations = [
+    return tuple(
         Violation(
             file=file_path,
             line=arg.annotation.lineno,
@@ -213,8 +276,7 @@ def _check_object_in_params(
             and arg.annotation.id == "object"
             and not has_suppression(content, arg.annotation.lineno, "ARCH009")
         )
-    ]
-    return tuple(violations)
+    )
 
 
 def _check_object_type_usage(node: ast.Name, file_path: Path, content: str) -> Violation | None:
@@ -246,14 +308,14 @@ def _check_object_type_usage(node: ast.Name, file_path: Path, content: str) -> V
     return None
 
 
-def _check_any_type_usage(node: ast.Name, file_path: Path) -> Violation | None:
+def _check_any_type_usage(node: ast.Name, file_path: Path, content: str) -> Violation | None:
     """Check for 'Any' type usage anywhere.
 
     Returns:
         Violation if Any type usage found, None otherwise.
 
     """
-    if node.id == "Any":
+    if node.id == "Any" and not has_suppression(content, node.lineno, "ARCH010"):
         return Violation(
             file=file_path,
             line=node.lineno,
@@ -293,20 +355,16 @@ def _check_import_from_node(
         Tuple of violations found.
 
     """
-    violations: list[Violation] = []
+    violations = ()
 
     # Check private imports
-    violation = _check_private_imports(node, file_path, is_internal=is_internal)
-    if violation:
-        violations.append(violation)
+    violations = (*violations, *_check_private_imports(node, file_path, is_internal=is_internal))
 
     # Check mock imports
-    violations.extend(_check_mock_imports(node, file_path))
+    violations = (*violations, *_check_mock_imports(node, file_path))
 
     # Check typing imports
-    violations.extend(_check_typing_imports(node, file_path, content))
-
-    return tuple(violations)
+    return (*violations, *_check_typing_imports(node, file_path, content))
 
 
 def _check_name_node(node: ast.Name, file_path: Path, content: str) -> tuple[Violation, ...]:
@@ -316,19 +374,19 @@ def _check_name_node(node: ast.Name, file_path: Path, content: str) -> tuple[Vio
         Tuple of violations found.
 
     """
-    violations: list[Violation] = []
+    violations = ()
 
     # Check object type usage
     violation = _check_object_type_usage(node, file_path, content)
     if violation:
-        violations.append(violation)
+        violations = (*violations, violation)
 
     # Check Any type usage
-    violation = _check_any_type_usage(node, file_path)
+    violation = _check_any_type_usage(node, file_path, content)
     if violation:
-        violations.append(violation)
+        violations = (*violations, violation)
 
-    return tuple(violations)
+    return violations
 
 
 def _check_if_node(node: ast.If, file_path: Path, content: str) -> tuple[Violation, ...]:
@@ -388,11 +446,11 @@ def check_file_imports(file_path: Path, content: str) -> tuple[Violation, ...]:
     # Check if this file is inside _internal (internal modules can import from each other)
     is_internal = "_internal" in str(file_path)
 
-    violations: list[Violation] = []
+    violations = ()
     for node in ast.walk(tree):
-        violations.extend(_process_node(node, file_path, content, is_internal=is_internal))
+        violations = (*violations, *_process_node(node, file_path, content, is_internal=is_internal))
 
-    return tuple(violations)
+    return violations
 
 
 def check_patch_decorators(file_path: Path, content: str) -> tuple[Violation, ...]:
@@ -402,8 +460,8 @@ def check_patch_decorators(file_path: Path, content: str) -> tuple[Violation, ..
         List of violations for improper @patch usage.
 
     """
-    violations: list[Violation] = []
-    lines = content.splitlines()
+    violations = ()
+    lines = tuple(content.splitlines())
 
     # Regex for @patch decorators
     patch_pattern = re.compile(r'@patch\(["\']([^"\']+)["\']\)')
@@ -413,7 +471,8 @@ def check_patch_decorators(file_path: Path, content: str) -> tuple[Violation, ..
         if match:
             target = match.group(1)
             if "clearflow" in target:
-                violations.append(
+                violations = (
+                    *violations,
                     Violation(
                         file=file_path,
                         line=line_num,
@@ -424,7 +483,7 @@ def check_patch_decorators(file_path: Path, content: str) -> tuple[Violation, ..
                     ),
                 )
 
-    return tuple(violations)
+    return violations
 
 
 def check_private_access(file_path: Path, content: str) -> tuple[Violation, ...]:
@@ -434,8 +493,8 @@ def check_private_access(file_path: Path, content: str) -> tuple[Violation, ...]
         List of violations for accessing private attributes.
 
     """
-    violations: list[Violation] = []
-    lines = content.splitlines()
+    violations = ()
+    lines = tuple(content.splitlines())
 
     # Check if this file is inside _internal (internal modules can access each other)
     is_internal = "_internal" in str(file_path)
@@ -460,7 +519,27 @@ def check_private_access(file_path: Path, content: str) -> tuple[Violation, ...]
             attr_name = match.group(1)
             # Skip dunder methods and internal module access
             if not attr_name.startswith("_") and not (is_internal and attr_name == "internal"):
-                violations.append(
+                # Check if this is same-module access (Pythonic convention)
+                # In Python, classes in the same module can access each other's private members
+                # Look for the pattern: object._method where object is likely a variable
+                # referring to another class instance in the same module
+
+                # Common patterns for same-module private access:
+                # - self._builder._add_route (accessing builder's private method)
+                # - return self._builder._add_termination (similar)
+                # These are valid when both classes are in the same file
+
+                # Check if this looks like accessing a private method of a collaborating class
+                # by looking for patterns like "variable._method" or "self._variable._method"
+                before_dot = line[: match.start()].rstrip()
+                if before_dot.endswith(("_builder", "_flow", "_node")):
+                    # This looks like same-module class collaboration
+                    # In clearflow/message_flow.py, _MessageFlowBuilderContext accessing
+                    # _FlowBuilder's methods is valid Python convention
+                    continue
+
+                violations = (
+                    *violations,
                     Violation(
                         file=file_path,
                         line=line_num,
@@ -471,7 +550,7 @@ def check_private_access(file_path: Path, content: str) -> tuple[Violation, ...]
                     ),
                 )
 
-    return tuple(violations)
+    return violations
 
 
 def check_file(file_path: Path) -> tuple[Violation, ...]:
@@ -481,7 +560,7 @@ def check_file(file_path: Path) -> tuple[Violation, ...]:
         List of all architecture violations found in the file.
 
     """
-    violations: list[Violation] = []
+    violations = ()
 
     try:
         content = file_path.read_text(encoding="utf-8")
@@ -494,11 +573,9 @@ def check_file(file_path: Path) -> tuple[Violation, ...]:
         return tuple(violations)
 
     # Run all checks
-    violations.extend(check_file_imports(file_path, content))
-    violations.extend(check_patch_decorators(file_path, content))
-    violations.extend(check_private_access(file_path, content))
-
-    return tuple(violations)
+    violations = (*violations, *check_file_imports(file_path, content))
+    violations = (*violations, *check_patch_decorators(file_path, content))
+    return (*violations, *check_private_access(file_path, content))
 
 
 def scan_directory(directory: Path) -> tuple[Violation, ...]:
@@ -508,7 +585,7 @@ def scan_directory(directory: Path) -> tuple[Violation, ...]:
         List of all violations found in Python files within the directory.
 
     """
-    violations: list[Violation] = []
+    violations = ()
 
     if not directory.exists():
         return tuple(violations)
@@ -520,9 +597,9 @@ def scan_directory(directory: Path) -> tuple[Violation, ...]:
         # Skip files in excluded directories
         if any(part in excluded_dirs for part in file_path.parts):
             continue
-        violations.extend(check_file(file_path))
+        violations = (*violations, *check_file(file_path))
 
-    return tuple(violations)
+    return violations
 
 
 def print_report(violations: tuple[Violation, ...]) -> None:
@@ -535,11 +612,11 @@ def print_report(violations: tuple[Violation, ...]) -> None:
     print("=" * 70)
 
     # Group by requirement
-    by_requirement: dict[str, list[Violation]] = {}
+    by_requirement = cast("dict[str, tuple[Violation, ...]]", {})
     for v in violations:
         if v.requirement not in by_requirement:
-            by_requirement[v.requirement] = []
-        by_requirement[v.requirement].append(v)
+            by_requirement[v.requirement] = ()
+        by_requirement[v.requirement] = (*by_requirement[v.requirement], v)
 
     for req, req_violations in sorted(by_requirement.items()):
         print(f"\n{req}: {len(req_violations)} violations")
@@ -572,23 +649,23 @@ def main() -> None:
     # Get items from command line arguments, or use defaults
     items = sys.argv[1:] if len(sys.argv) > 1 else ["clearflow", "tests", "examples"]
 
-    all_violations: list[Violation] = []
+    all_violations = ()
     for item in items:
         path = Path(item)
         if path.is_file() and path.suffix == ".py":
             # Single Python file
             violations = check_file(path)
-            all_violations.extend(violations)
+            all_violations = (*all_violations, *violations)
         elif path.is_dir():
             # Directory - scan recursively
             violations = scan_directory(path)
-            all_violations.extend(violations)
+            all_violations = (*all_violations, *violations)
         else:
             print(f"Warning: {item} is not a Python file or directory, skipping")
             continue
 
     # Print report
-    print_report(tuple(all_violations))
+    print_report(all_violations)
 
     # Exit with appropriate code
     if all_violations:
